-- -------------------------------------------------------------------------- --
-- AHsearch by kunda                                                          --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- AHsearch is a World of Warcraft AddOn that saves Auction House search      --
-- queries.                                                                   --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Features:                                                                  --
-- AHsearch puts a 16x16 pixel 'Arrow Down' button ('AHsearch Button') on the --
-- right side of your searchfield in the Auction House frame.                 --
--                                                                            --
-- - Right-click 'AHsearch Button' for options like create, edit, delete,     --
--   sort, move categories/search queries.                                    --
-- - Left-click 'AHsearch Button' for your saved search queries.              --
-- - Cycle Search: (next/previous/random)                                     --
--   - Key Bindings [ESC -> Key bindings -> AHsearch]                         --
--   - Buttons [AHsearch options menu -> Options -> Buttons]                  --
--                                                                            --
-- How To:                                                                    --
-- Step 1 or 2: Right-click 'AHsearch Button' and create category             --
--              ('Category: Create').                                         --
-- Step 1 or 2: Set up your search criteria (you don't have to click the      --
--              search button).                                               --
-- Step 3: Right-click 'AHsearch Button' and choose a category in             --
--         'Search query: Save'.                                              --
-- Step 4: Give your search a name. Press Enter.                              --
-- Step 5: Left-click 'AHsearch Button' for your search queries.              --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Credits:                                                                   --
-- idea based on AHFavorites from Narwick                                     --
-- cycle search idea based on AHsearchNext from Evan                          --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
AHsearch = CreateFrame("Frame")              -- container
AHsearch_Data = {}                           -- SavedVariable Data table
AHsearch_Options = {}                        -- SavedVariable Options table

local L = AHsearch_Locales                   -- localization table

local _G = _G
local string_format = _G.string.format
local table_insert = _G.table.insert
local table_sort = _G.table.sort

local Dewdrop = AceLibrary("Dewdrop-2.0")    -- Dewdrop-2.0 (requires AceLibrary)
local maxBoxInputLength = 40                 -- maximum characters for box entries
local maxItemLevel = 80                      -- maximum required ItemLevel (maximum level a character can reach)
local cycle                                  -- nil/false = no key binding action executed | true = key binding allowed 
local cylceCat                               -- category for cycle search
local cycleSQ                                -- search query for cycle search

local colorBase        = "|cffffff7f"
local colorAction      = "|cffff9900"
local colorTooltipDesc = "|cffbfb166"
local colorTooltipData = "|cffffffff"
local colorTooltipYes  = "|cff00ff00"
local colorTooltipNo   = "|cffff0000"

local itemRarityColors = {
	[1] = "|cffffff9a",       -- all (AH: -1)
	[2] = "|cff9d9d9d",       -- poor (AH: 0)
	[3] = "|cffffffff",       -- common (AH: 1)
	[4] = "|cff1eff00",       -- uncommon (AH: 2)
	[5] = "|cff0070dd",       -- rare (AH: 3)
	[6] = "|cffa335ee"        -- epic (AH: 4)
--[7] = "|cffff8000",       -- legendary (not in AH index)
--[8] = "|cffe6cc80",       -- artifact (not in AH index)
}
local itemRarityNames = {
	[1] = L["All"],           -- all (AH: -1)
	[2] = ITEM_QUALITY0_DESC, -- poor (AH: 0)
	[3] = ITEM_QUALITY1_DESC, -- common (AH: 1)
	[4] = ITEM_QUALITY2_DESC, -- uncommon (AH: 2)
	[5] = ITEM_QUALITY3_DESC, -- rare (AH: 3)
	[6] = ITEM_QUALITY4_DESC  -- epic (AH: 4)
--[7] =                     -- legendary (not in AH index)
--[8] =                     -- artifact (not in AH index)
}
local categoryColors = {
	[1] = "|cffffffff",
	[2] = "|cffbbbbbb",
	[3] = "|cff777777",
	[4] = "|cfff32f00",
	[5] = "|cffff8c00",
	[6] = "|cffffd700",
	[7] = "|cffdeb887",
	[8] = "|cffa0522d",
	[9] = "|cffe9967a",
	[10] = "|cff4169e1",
	[11] = "|cff008cb4",
	[12] = "|cff7edef4",
	[13] = "|cffa020f0",
	[14] = "|cffee82ee",
	[15] = "|cff95004a",
	[16] = "|cff00ff00"
}
local categoryIcons = {
	[1] = "",
	[2] = "Interface\\Icons\\INV_Scroll_03",
	[3] = "Interface\\Icons\\INV_Misc_Bag_10_Black",
	[4] = "Interface\\Icons\\INV_Misc_Coin_01",
	[5] = "Interface\\Icons\\INV_Jewelry_Talisman_08",
	[6] = "Interface\\Icons\\INV_Misc_Bomb_04",
	[7] = "Interface\\Icons\\INV_Misc_Gem_Pearl_04",
	[8] = "Interface\\Icons\\INV_Misc_Gem_Sapphire_01",
	[9] = "Interface\\Icons\\INV_Misc_Gift_01",
	[10] = "Interface\\GroupFrame\\UI-Group-LeaderIcon",
	[11] = "Interface\\Icons\\INV_Potion_03",
	[12] = "Interface\\Icons\\Trade_BlackSmithing",
	[13] = "Interface\\Icons\\Trade_Engineering",
	[14] = "Interface\\Icons\\Trade_Engraving",
	[15] = "Interface\\Icons\\Trade_Mining",
	[16] = "Interface\\Icons\\Trade_Tailoring",
	[17] = "Interface\\Icons\\INV_Misc_Gem_01",
	[18] = "Interface\\Icons\\INV_Misc_Food_15",
	[19] = "Interface\\Icons\\Spell_Holy_SealOfSacrifice",
	[20] = "Interface\\Icons\\Spell_Holy_MindVision"
}
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- DB Version Check
-- --------------------------
local function DBcheck()
	if AHsearchDB then
		--DEFAULT_CHAT_FRAME:AddMessage("AHsearchDB exixts")
		if AHsearchDB.profiles then
			-- check if multiple profiles exists START
			local x = 0
			for k,v in pairs(AHsearchDB.profiles) do
				if k then
					x = x + 1
				end
			end
			if x > 1 then
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r "..colorAction..x.."|r old AHsearch AceDB-2.0 profiles detected.")
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r Only the profile 'Default' (if exists) is copied to the new DB format.")
			end
			-- check if multiple profiles exists END

			if AHsearchDB.profiles.Default then
				-- copy profile 'Default' AHsearchDB to AHsearch_Data START
				local x = 0
				for k,v in pairs(AHsearchDB.profiles.Default.search) do
					if not AHsearch_Data[k] then
						AHsearch_Data[k] = {}
						AHsearch_Data[k] = v
						x = x + 1
					end
				end
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r "..colorAction..x.."|r categories copied to new DB format.")
				-- copy profile 'Default' AHsearchDB to AHsearch_Data END
				-- check if color and icon exists (db version < 1). if not create dummy entry START
				for k,v in pairs(AHsearch_Data) do
					if not AHsearch_Data[k].color then
						AHsearch_Data[k].color = "|cffffffff"
						AHsearch_Data[k].icon = ""
					end
				end
				-- check if color and icon exists (db version < 1). if not create dummy entry END
				-- copy profile 'Default' AHsearchDB options to AHsearch_Options START
				if AHsearchDB.profiles.Default.chatOutput then
					AHsearch_Options.chatOutput = AHsearchDB.profiles.Default.chatOutput
				end
				if AHsearchDB.profiles.Default.ttDisplay then
					AHsearch_Options.ttDisplay = AHsearchDB.profiles.Default.ttDisplay
				end
				if AHsearchDB.profiles.Default.ttFiltersDisplay then
					AHsearch_Options.ttFiltersDisplay = AHsearchDB.profiles.Default.ttFiltersDisplay
				end		
				-- copy profile 'Default' AHsearchDB options to AHsearch_Options END
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r No profile 'Default' detected. No old AHsearch AceDB-2.0 data saved!")
			end
		end
		AHsearchDB = nil -- delete old db
	end

	if not AHsearch_Options.noWotLKmessage then
		local x
		for k,v in pairs(AHsearch_Data) do
			x = true
			break
		end
		if x then
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r WoW Client 3.x detected. 3.x changed some Auction House Filters.")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r If you want AHsearch to automatically update your saved data type:")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r "..colorAction.."/ahsearch wotlk|r")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r Please read Changelog File (r54 - 2008-09-26) for more information! Use this command carefully!")
		end
		AHsearch_Options.noWotLKmessage = true
	end
end

local function FixWotLKCategories()
	if not AHsearch_Options.WotLKfix then
		local changedIndex = 0
		local invalidIndex = 0
		for k,v in pairs(AHsearch_Data) do
			for k2,v2 in pairs(AHsearch_Data[k].queries) do
				if AHsearch_Data[k].queries[k2].classIndex then
					if AHsearch_Data[k].queries[k2].classIndex >= 5 then
						local newIndex = AHsearch_Data[k].queries[k2].classIndex + 1
						if newIndex <= 12 then
							AHsearch_Data[k].queries[k2].classIndex = newIndex
							changedIndex = changedIndex + 1
						else
							invalidIndex = invalidIndex + 1
						end
					end
				end
			end
		end
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r "..colorAction..changedIndex.."|r search queries corrected.")
		if invalidIndex > 0 then
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch:|r "..colorAction..invalidIndex.."|r search queries with index > 12 detected.")
		end
		AHsearch_Options.WotLKfix = true
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- local functions
-- --------------------------
local function Print(text)
	if AHsearch_Options.chatOutput and DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff7fAHsearch: |r"..text)
	end
end

local function SortedPairs(t,lvl)
	local ntab = {}
	for k,v in pairs(t) do
		table_insert(ntab,k)
	end

	if lvl then
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if t[a].qualityIndex == t[b].qualityIndex then
						if a < b then
							return true
						end
					end
					if t[a].qualityIndex > t[b].qualityIndex then
						return true
					end	
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	else
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if a < b then
						return true
					end
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	end

	local i = 0
	local function _f(_s,_v)
		i = i + 1
		local k = ntab[i]
		if (k) then
			return k,t[k]
		end
	end
	return _f,nil,nil
end

local function GCheck()
	local x = true
	for k,v in pairs(AHsearch_Data) do
		for l,w in pairs(AHsearch_Data[k].queries) do
			x = false
			break
		end
	end
	return x
end

local function CatCheck()
	local x = true
	for k,v in pairs(AHsearch_Data) do
		x = false
		break
	end
	return x
end

local function CatCount()
	local x = 0
	for k,v in pairs(AHsearch_Data) do
		x = x + 1
	end
	return x
end

local function SQCheck(cat)
	local x = true
	for k,v in pairs(AHsearch_Data[cat].queries) do
		x = false
		break
	end
	return x
end

local function SQCount(cat)
	local x = 0
	for k,v in pairs(AHsearch_Data[cat].queries) do
		x = x + 1
	end
	return x
end

local function ConfirmCheck(t,check)
	local x = false
	for k,v in pairs(t) do
		if k == check then
			x = true
			break
		end
	end
	return x
end

local function TooltipCheck(text)
	if AHsearch_Options.ttDisplay then
		return text
	else
		return ""
	end
end

local function QualityColored(cat,sq)
	return itemRarityColors[(AHsearch_Data[cat].queries[sq].qualityIndex+2)]..sq
end

local function SlashHandler(msg)
	msg = strtrim(string.lower(msg or ""))
	if not AHsearch_Options.WotLKfix and msg == "wotlk" then
		FixWotLKCategories()
	else
		AHsearch:HelpOutput()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:InitOptions()
	SlashCmdList["AHSEARCH"] = SlashHandler
	SLASH_AHSEARCH1 = "/ahsearch"

	if AHsearch_Options.version              == nil then AHsearch_Options.version              = 3.1   end
	if AHsearch_Options.version              == 3   then AHsearch_Options.version              = 3.1   end
	
	if AHsearch_Options.chatOutput           == nil then AHsearch_Options.chatOutput           = false end
	if AHsearch_Options.ttDisplay            == nil then AHsearch_Options.ttDisplay            = false end
	if AHsearch_Options.ttFiltersDisplay     == nil then AHsearch_Options.ttFiltersDisplay     = true  end
	if AHsearch_Options.cycleButtonsNextPrev == nil then AHsearch_Options.cycleButtonsNextPrev = false end
	if AHsearch_Options.cycleButtonsRandom   == nil then AHsearch_Options.cycleButtonsRandom   = false end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CreateAHsearchButton()
	local f = CreateFrame("Frame", "AHsearchFrame", AuctionFrameBrowse)
	f:SetFrameLevel(AuctionFrameBrowse:GetFrameLevel()+10)
	f:SetWidth(16)
	f:SetHeight(16)
	f:ClearAllPoints()
	f:SetPoint("RIGHT", "BrowseName", "RIGHT", 0, 0)
	local texture = f:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(f)
	texture:SetTexture(0, 0, 0, 1)
	local b = CreateFrame("Button", "AHsearchButton", AHsearchFrame)
	b:SetFrameLevel(AHsearchFrame:GetFrameLevel()+1)
	b:SetWidth(16)
	b:SetHeight(16)
	b:ClearAllPoints()
	b:SetPoint("RIGHT", "AHsearchFrame", "RIGHT", 0, 0)
	local normalTexture = b:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(b)
	normalTexture:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	normalTexture:SetTexCoord(4/32,27/32,5/32,27/32)
	b:SetNormalTexture(normalTexture)
	local highlightTexture = b:CreateTexture(nil, "BACKGROUND")
	highlightTexture:SetAllPoints(b)
	highlightTexture:SetTexture("Interface\\Buttons\\UI-Common-MouseHilight")
	highlightTexture:SetTexCoord(5/32,27/32,5/32,27/32)
	b:SetHighlightTexture(highlightTexture)
	local pushedTexture = b:CreateTexture(nil, "BACKGROUND")
	pushedTexture:SetAllPoints(b)
	pushedTexture:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	pushedTexture:SetTexCoord(4/32,27/32,5/32,27/32)
	b:SetPushedTexture(pushedTexture)
	b:SetScript("OnClick", function() AHsearch:CreatePullDownMenu() end)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CreateAHsearchCycleButtons()
	if AHsearch_Options.cycleButtonsNextPrev then
		if not AHsearchCycleFramePrev then
			local f = CreateFrame("Frame", "AHsearchCycleFramePrev", AHsearchFrame)
			f:SetFrameLevel(AuctionFrameBrowse:GetFrameLevel()+10)
			f:SetWidth(14)
			f:SetHeight(14)
			f:ClearAllPoints()
			f:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 0, 0)
			local texture = f:CreateTexture(nil, "BACKGROUND")
			texture:SetAllPoints(f)
			texture:SetTexture(0, 0, 0, 1)
			local b = CreateFrame("Button", "AHsearchCycleButtonPrev", AHsearchCycleFramePrev)
			b:SetFrameLevel(AHsearchCycleFramePrev:GetFrameLevel()+1)
			b:SetWidth(14)
			b:SetHeight(14)
			b:SetAllPoints(f)
			local normalTexture = b:CreateTexture(nil, "BACKGROUND")
			normalTexture:SetAllPoints(b)
			normalTexture:SetTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up")
			normalTexture:SetTexCoord(4/32,27/32,5/32,27/32)
			local highlightTexture = b:CreateTexture(nil, "BACKGROUND")
			highlightTexture:SetAllPoints(b)
			highlightTexture:SetTexture("Interface\\Buttons\\UI-Common-MouseHilight")
			highlightTexture:SetTexCoord(5/32,27/32,5/32,27/32)
			b:SetNormalTexture(normalTexture)
			b:SetHighlightTexture(highlightTexture)
			--b:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down")
			b:SetScript("OnClick", function() AHsearch:CycleSearch(-1) end)
			
			local f = CreateFrame("Frame", "AHsearchCycleFrameNext", AHsearchFrame)
			f:SetFrameLevel(AuctionFrameBrowse:GetFrameLevel()+10)
			f:SetWidth(14)
			f:SetHeight(14)
			f:ClearAllPoints()
			f:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", 0, 0)
			local texture = f:CreateTexture(nil, "BACKGROUND")
			texture:SetAllPoints(f)
			texture:SetTexture(0, 0, 0, 1)
			local b = CreateFrame("Button", "AHsearchCycleButtonNext", AHsearchCycleFrameNext)
			b:SetFrameLevel(AHsearchCycleFrameNext:GetFrameLevel()+1)
			b:SetWidth(14)
			b:SetHeight(14)
			b:SetAllPoints(f)
			local normalTexture = b:CreateTexture(nil, "BACKGROUND")
			normalTexture:SetAllPoints(b)
			normalTexture:SetTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
			normalTexture:SetTexCoord(4/32,27/32,5/32,27/32)
			local highlightTexture = b:CreateTexture(nil, "BACKGROUND")
			highlightTexture:SetAllPoints(b)
			highlightTexture:SetTexture("Interface\\Buttons\\UI-Common-MouseHilight")
			highlightTexture:SetTexCoord(5/32,27/32,5/32,27/32)
			b:SetNormalTexture(normalTexture)
			b:SetHighlightTexture(highlightTexture)
			--b:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
			b:SetScript("OnClick", function() AHsearch:CycleSearch(1) end)
		end
	end
	if AHsearch_Options.cycleButtonsRandom then
		if not AHsearchCycleFrameRandom then
			local f = CreateFrame("Frame", "AHsearchCycleFrameRandom", AHsearchFrame)
			f:SetFrameLevel(AuctionFrameBrowse:GetFrameLevel()+10)
			f:SetWidth(14)
			f:SetHeight(14)
			f:ClearAllPoints()
			f:SetPoint("TOP", "AHsearchFrame", "BOTTOM", 0, 0)
			local texture = f:CreateTexture(nil, "BACKGROUND")
			texture:SetAllPoints(f)
			texture:SetTexture(0, 0, 0, 1)
			local b = CreateFrame("Button", "AHsearchCycleButtonRandom", AHsearchCycleFrameRandom)
			b:SetFrameLevel(AHsearchCycleFrameRandom:GetFrameLevel()+1)
			b:SetWidth(14)
			b:SetHeight(14)
			b:SetAllPoints(f)
			local normalTexture = b:CreateTexture(nil, "BACKGROUND")
			normalTexture:SetAllPoints(b)
			normalTexture:SetTexture("Interface\\Buttons\\UI-RotationLeft-Button-Up")
			normalTexture:SetTexCoord(4/32,27/32,5/32,27/32)
			b:SetNormalTexture(normalTexture)
			b:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
			--b:SetPushedTexture("Interface\\Buttons\\UI-RotationLeft-Button-Down")
			b:SetScript("OnClick", function() AHsearch:RandomSearch() end)
		end
	end
	if AHsearchCycleFramePrev then
		if AHsearch_Options.cycleButtonsRandom then
			AHsearchCycleFramePrev:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 0, 0)
			AHsearchCycleFrameNext:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", 0, 0)
		else
			AHsearchCycleFramePrev:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 6, 0)
			AHsearchCycleFrameNext:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", -6, 0)
		end
	end
end

function AHsearch:ToggleCycleButtons(button)
	if button == "np" then
		AHsearch:CreateAHsearchCycleButtons()
		if AHsearch_Options.cycleButtonsNextPrev then
			AHsearchCycleFrameNext:Show()
			AHsearchCycleFramePrev:Show()
		else
			AHsearchCycleFrameNext:Hide()
			AHsearchCycleFramePrev:Hide()
		end
	end
	if button == "rnd" then
		AHsearch:CreateAHsearchCycleButtons()
		if AHsearch_Options.cycleButtonsRandom then
			AHsearchCycleFrameRandom:Show()
		else
			AHsearchCycleFrameRandom:Hide()
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CreatePullDownMenu()
	local rcc = CatCheck()
	Dewdrop:Open(AHsearchFrame,
		'point', function(parent)
			return "TOPLEFT","BOTTOMLEFT"
		end,
		'dontHook', true,
		'children', function(level, value)
			if rcc then
				if level == 1 then
					Dewdrop:AddLine(
						'isTitle', true,
						'text', L["No category available!"]
					)
					Dewdrop:AddLine(
						'isTitle', true,
						'text', L["Right-click 'AHsearch Button' for options!"]
					)
				end
			else
				if level == 1 then
					for cat,v in SortedPairs(AHsearch_Data) do
						local x = SQCheck(cat)
						Dewdrop:AddLine(
							'text', AHsearch_Data[cat].color..cat,
							'tooltipTitle', TooltipCheck(AHsearch_Data[cat].color..cat),
							'icon', AHsearch_Data[cat].icon,
							'hasArrow', not x,
							'disabled', false,
							'notClickable', x,
							'value', {'catname', cat}
						)
					end
				elseif level == 2 then
					if type(value) == "table" and value[1] == "catname" then
						local cat = value[2]
						for sq,v in SortedPairs(AHsearch_Data[cat].queries,1) do
							Dewdrop:AddLine(
								'text', QualityColored(cat,sq),
								'tooltipTitle', TooltipCheck(L["Search"]),
								'tooltipText', self:SQtooltip(cat,sq),
								'func', function()
									self:Search(cat,sq)
								end
							)
						end
					end
				end
			end
		end
	)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CreatePullDownOptionsMenu()
	local tab = AHsearch_Data
	Dewdrop:Register(AHsearchButton,
		'point', function(parent)
			return "TOPLEFT","BOTTOMLEFT"
		end,
		'children', function(level, value)
			-- ----------------------------------------------------------------------
			if level == 1 then
			-- ----------------------------------------------------------------------
				local x = GCheck()
				local y = CatCheck()
				Dewdrop:AddLine(
					'isTitle', true,
					'text', L["Search query"]
				)
				Dewdrop:AddLine(
					'text', L["Search query: Save"],
					'tooltipTitle', TooltipCheck(L["Search query: Save"]),
					'tooltipText', L["Save your current search query in a category."],
					'disabled', y,
					'hasArrow', not y,
					'value', "saveSearchQuery"
				)
				Dewdrop:AddLine(
					'text', L["Search query: Edit"],
					'tooltipTitle', TooltipCheck(L["Search query: Edit"]),
					'tooltipText', L["Edit position, search query, search term, level range, rarity, usable items, display on character and filters for a search query."],
					'disabled', x,
					'hasArrow', not x,
					'value', "editSearchQuery"
				)
				Dewdrop:AddLine(
					'text', L["Search query: Move"],
					'tooltipTitle', TooltipCheck(L["Search query: Move"]),
					'tooltipText', L["Move a search query from one category to another category.\n\nATTENTION:\nPosition is set to 1!"],
					'disabled', x,
					'hasArrow', not x,
					'value', "moveSearchQuery"
				)
				Dewdrop:AddLine(
					'text', L["Search query: Delete"],
					'tooltipTitle', TooltipCheck(L["Search query: Delete"]),
					'tooltipText', L["Delete a search query."],
					'disabled', x,
					'hasArrow', not x,
					'value', "deleteSearchQuery"
				)
				Dewdrop:AddLine(
					'isTitle', true,
					'text', L["Category"]
				)
				Dewdrop:AddLine(
					'text', L["Category: Create"],
					'tooltipTitle', TooltipCheck(L["Category: Create"]),
					'tooltipText', L["Create a new category."],
					'hasArrow', true,
					'hasEditBox', true,
					'editBoxValidateFunc', function(catname)
						if catname:len() >= 1 and catname:len() <= maxBoxInputLength then
							return true
						end
					end,
					'editBoxFunc', function(catname)
						local owCheck = ConfirmCheck(tab,catname)
						local func = function()
							tab[catname] = {}
							tab[catname].order = 1
							tab[catname].color = "|cffffffff"
							tab[catname].icon = ""
							tab[catname].queries = {}
							Print(string_format(L["Category '%s' CREATED!"],catname))
						end
						if owCheck then
							local confirmTxt = string_format(L["Category '%s' exists!\nOverwrite excisting category '%s'?\nAll search queries in that category will be deleted!"],catname,catname)
							self:ConfirmPopup(confirmTxt,func)
						else
							func()
						end
					end
				)
				Dewdrop:AddLine(
					'text', L["Category: Edit"],
					'tooltipTitle', TooltipCheck(L["Category: Edit"]),
					'tooltipText', L["Rename, sort, set color and icon for a category."],
					'disabled', y,
					'hasArrow', not y,
					'value', "editCat"
				)
				Dewdrop:AddLine(
					'text', L["Category: Delete"],
					'tooltipTitle', TooltipCheck(L["Category: Delete"]),
					'tooltipText', L["Delete category and all saved search queries in that category!"],
					'disabled', y,
					'hasArrow', not y,
					'value', "deleteCat"
				)
				Dewdrop:AddLine(
					'isTitle', true,
					'text', L["Options"]
				)
				Dewdrop:AddLine(
					'text', L["Options"],
					'tooltipTitle', TooltipCheck(L["Options"]),
					'hasArrow', true,
					'value', "options"
				)
				Dewdrop:AddLine(
					'text', L["Chatoutput"],
					'tooltipTitle', TooltipCheck(L["Chatoutput"]),
					'tooltipText', L["Activate/Deactivate chatoutput for:\n- SEARCH\n- Search query: SAVE, EDIT, MOVE & DELETE\n- Category: CREATE, EDIT & DELETE"],
					'checkIcon', "Interface\\Buttons\\UI-CheckBox-Check",
					'checked', AHsearch_Options.chatOutput,
					'func', function()
						AHsearch_Options.chatOutput = not AHsearch_Options.chatOutput
					end
				)
				Dewdrop:AddLine(
					'text', L["Help"],
					'tooltipTitle', TooltipCheck(L["Help"]),
					'func', function()
						self:HelpOutput()
					end
				)
			-- ----------------------------------------------------------------------
			elseif level == 2 then
			-- ----------------------------------------------------------------------
				if value == "saveSearchQuery" then
					for cat,v in SortedPairs(tab) do
						Dewdrop:AddLine(
							'text', tab[cat].color..cat,
							'tooltipTitle', TooltipCheck(L["Search query: Save"]),
							'tooltipText', self:TooltipCATName(cat),
							'icon', tab[cat].icon,
							'hasArrow', true,
							'hasEditBox', true,
							'editBoxText', BrowseName:GetText() or "",
							'editBoxValidateFunc', function(sname)
								if sname:len() >= 1 and sname:len() <= maxBoxInputLength then
									return true
								end
							end,
							'editBoxFunc', function(sname)
								local owCheck = ConfirmCheck(tab[cat].queries,sname)
								local func = function()
									self:SaveSearchInCategory(cat,sname)
								end
								if owCheck then
									local confirmTxt = string_format(L["Overwrite excisting search query '%s' in category '%s'?"],sname,cat)
									self:ConfirmPopup(confirmTxt,func)
								else
									func()
								end
							end
						)
					end
				end

				if value == "editSearchQuery" then
					for cat,v in SortedPairs(tab) do
						local x = SQCheck(cat)
						Dewdrop:AddLine(
							'text', tab[cat].color..cat,
							'tooltipTitle', TooltipCheck(L["Search query: Edit"]),
							'tooltipText', self:TooltipCATName(cat),
							'icon', tab[cat].icon,
							'hasArrow', not x,
							'disabled', false,
							'notClickable', x,
							'value', {'editSearchQuery', cat}
						)
					end
				end

				if value == "deleteSearchQuery" then
					for cat,v in SortedPairs(tab) do
						local x = SQCheck(cat)
						Dewdrop:AddLine(
							'text', tab[cat].color..cat,
							'tooltipTitle', TooltipCheck(L["Search query: Delete"]),
							'tooltipText', self:TooltipCATName(cat),
							'icon', tab[cat].icon,
							'hasArrow', not x,
							'disabled', false,
							'notClickable', x,
							'value', {'deleteSearchQuery', cat}
						)
					end
				end

				if value == "moveSearchQuery" then
					for cat,v in SortedPairs(tab) do
						local x = SQCheck(cat)
						Dewdrop:AddLine(
							'text', tab[cat].color..cat,
							'tooltipTitle', TooltipCheck(L["Search query: Move"]),
							'tooltipText', self:TooltipCATName(cat),
							'icon', tab[cat].icon,
							'hasArrow', not x,
							'disabled', false,
							'notClickable', x,
							'value', {'moveSearchQuery', cat}
						)
					end
				end

				if value == "editCat" then
					for cat,v in SortedPairs(tab) do
						Dewdrop:AddLine(
							'text', tab[cat].order.." - "..tab[cat].color..cat..itemRarityColors[2].." ("..SQCount(cat)..")",
							'tooltipTitle', TooltipCheck(L["Category: Edit"]),
							'tooltipText', self:CATtooltip(cat),
							'icon', tab[cat].icon,
							'hasArrow', true,
							'value', {'editCat2', cat}
						)
					end
				end

				if value == "deleteCat" then
					for cat,v in SortedPairs(tab) do
						Dewdrop:AddLine(
							'text', tab[cat].color..cat..itemRarityColors[2].." ("..SQCount(cat).." "..L["search queries"]..")",
							'tooltipTitle', TooltipCheck(L["Category: Delete"]),
							'tooltipText', colorTooltipDesc..L["Category"]..": "..tab[cat].color..cat.."\n"..itemRarityColors[2].."("..SQCount(cat).." "..L["search queries"]..")",
							'icon', tab[cat].icon,
							'func', function()
								local confirmTxt = string_format(L["Delete category '%s'?\nAll search queries in that category will be deleted!"],cat)
								local func = function()
									tab[cat] = nil
									Print(string_format(L["Category '%s' DELETED!"],cat))
								end
								self:ConfirmPopup(confirmTxt,func)
								Dewdrop:Refresh(2)
							end
						)
					end
				end

				if value == "options" then
					Dewdrop:AddLine(
						'text', L["Tooltip"],
						'tooltipTitle', TooltipCheck(L["Tooltip settings"]),
						'tooltipText', L["Tooltip settings"]..".",
						'hasArrow', true,
						'value', {'tooltip'}
					)
					Dewdrop:AddLine(
						'text', L["Buttons"],
						'tooltipTitle', TooltipCheck(L["Button settings"]),
						'tooltipText', L["Button settings"]..".",
						'hasArrow', true,
						'value', {'buttons'}
					)
				end
			-- ----------------------------------------------------------------------
			elseif level == 3 then
			-- ----------------------------------------------------------------------
				if type(value) == "table" and value[1] == "editSearchQuery" then
					local subTab = value[2]
					for sq,v in SortedPairs(tab[subTab].queries,1) do
						Dewdrop:AddLine(
							'text', tab[subTab].queries[sq].order.." - "..QualityColored(subTab,sq),
							'tooltipTitle', TooltipCheck(L["Search query: Edit"]),
							'tooltipText', self:SQtooltip(subTab,sq),
							'hasArrow', true,
							'value', {'editSearchQuery2', subTab, sq}
						)
					end
				end

				if type(value) == "table" and value[1] == "deleteSearchQuery" then
					local subTab = value[2]
					for sq,v in SortedPairs(tab[subTab].queries,1) do
						Dewdrop:AddLine(
							'text', tab[subTab].queries[sq].order.." - "..QualityColored(subTab,sq),
							'tooltipTitle', TooltipCheck(L["Search query: Delete"]),
							'tooltipText', self:SQtooltip(subTab,sq),
							'func', function()
								local confirmTxt = string_format(L["Delete search query '%s' in category '%s'?"],sq,subTab)
								local func = function()
									tab[subTab].queries[sq] = nil
									Print(string_format(L["Search query '%s' in category '%s' DELETED!"],sq,subTab))
								end
								self:ConfirmPopup(confirmTxt,func)
							end
						)
					end
				end

				if type(value) == "table" and value[1] == "moveSearchQuery" then
					local subTab = value[2]
					for sq,v in SortedPairs(tab[subTab].queries,1) do
						Dewdrop:AddLine(
							'text', tab[subTab].queries[sq].order.." - "..QualityColored(subTab,sq),
							'tooltipTitle', TooltipCheck(L["Search query: Move"]),
							'tooltipText', self:SQtooltip(subTab,sq),
							'hasArrow', true,
							'value', {'moveSearchQuery2', subTab, sq}
						)
					end
				end

				if type(value) == "table" and value[1] == "editCat2" then
					local cat = value[2]
					Dewdrop:AddLine(
						'text', self:TooltipCATName(cat),
						'tooltipTitle', TooltipCheck(L["Category: Rename"]),
						'tooltipText', L["Rename category."],
						'hasArrow', true,
						'hasEditBox', true,
						'editBoxText', cat,
						'editBoxValidateFunc', function(newCatName)
							if newCatName:len() >= 1 and newCatName:len() <= maxBoxInputLength then
								return true
							end
						end,
						'editBoxFunc', function(newCatName)
							if newCatName ~= cat then
								local owCheck = ConfirmCheck(tab,newCatName)
								local func = function()
									tab[newCatName] = tab[cat]
									tab[cat] = nil
									Print(string_format(L["Category '%s' RENAMED TO '%s'!"],cat,newCatName))
								end
								if owCheck then
									local confirmTxt = string_format(L["Category '%s' exists!\nOverwrite excisting category '%s'?\nAll search queries in that category will be deleted!"],newCatName,newCatName)
									self:ConfirmPopup(confirmTxt,func)
								else
									func()
								end
								Dewdrop:Refresh(2)
								Dewdrop:Close(3)
								Dewdrop:Close(4)
							end
						end
					)
					Dewdrop:AddLine(
						'text', self:TooltipCATPosition(cat),
						'tooltipTitle', TooltipCheck(L["Category: Sort"]),
						'tooltipText', L["Sort category by assigning a number to it.\nSort sequence:\n1.) number\n2.) upper case\n3.) lower case"],
						'disabled', CatCheck(),
						'hasArrow', true,
						'value', {'editCat3pos', cat}
					)
					Dewdrop:AddLine(
						'text', self:TooltipCATColor(cat),
						'tooltipTitle', TooltipCheck(L["Category: Color"]),
						'tooltipText', L["Set color for a category."],
						'disabled', CatCheck(),
						'hasArrow', true,
						'value', {'editCat3color', cat}
					)
					Dewdrop:AddLine(
						'text', self:TooltipCATIcon(cat),
						'tooltipTitle', TooltipCheck(L["Category: Icon"]),
						'tooltipText', L["Set icon for a category."],
						'icon', tab[cat].icon,
						'disabled', CatCheck(),
						'hasArrow', true,
						'value', {'editCat3icon', cat}
					)
				end

				if type(value) == "table" and value[1] == "tooltip" then
					Dewdrop:AddLine(
						'text', L["Show tooltip"],
						'tooltipTitle', TooltipCheck(L["Show tooltip"]),
						'tooltipText', L["Show tooltip"]..".",
						'checkIcon', "Interface\\Buttons\\UI-CheckBox-Check",
						'checked', AHsearch_Options.ttDisplay,
						'func', function()
							AHsearch_Options.ttDisplay = not AHsearch_Options.ttDisplay
						end
					)
					Dewdrop:AddLine(
						'text', L["Simple filters display"],
						'tooltipTitle', TooltipCheck(L["Simple filters display"]),
						'tooltipText', L["Tooltip shows only selected filters resp. 'no filters' instead of\n'1.) -'\n'2.) -'\n'3.) -'"],
						'checkIcon', "Interface\\Buttons\\UI-CheckBox-Check",
						'checked', AHsearch_Options.ttFiltersDisplay,
						'disabled', not AHsearch_Options.ttDisplay,
						'func', function()
							AHsearch_Options.ttFiltersDisplay = not AHsearch_Options.ttFiltersDisplay
						end
					)
				end

				if type(value) == "table" and value[1] == "buttons" then
					Dewdrop:AddLine(
						'text', L["Show Button for Next and Previous Search"],
						'tooltipTitle', TooltipCheck(L["Show Button for Next and Previous Search"]),
						'tooltipText', L["Show Button for Next and Previous Search"]..".",
						'checkIcon', "Interface\\Buttons\\UI-CheckBox-Check",
						'checked', AHsearch_Options.cycleButtonsNextPrev,
						'func', function()
							AHsearch_Options.cycleButtonsNextPrev = not AHsearch_Options.cycleButtonsNextPrev
							AHsearch:ToggleCycleButtons("np")
						end
					)
					Dewdrop:AddLine(
						'text', L["Show Button for Random Search"],
						'tooltipTitle', TooltipCheck(L["Show Button for Random Search"]),
						'tooltipText', L["Show Button for Random Search"]..".",
						'checkIcon', "Interface\\Buttons\\UI-CheckBox-Check",
						'checked', AHsearch_Options.cycleButtonsRandom,
						'func', function()
							AHsearch_Options.cycleButtonsRandom = not AHsearch_Options.cycleButtonsRandom
							AHsearch:ToggleCycleButtons("rnd")
						end
					)
				end

			-- ----------------------------------------------------------------------
			elseif level == 4 then
			-- ----------------------------------------------------------------------
				if type(value) == "table" and value[1] == "editSearchQuery2" then
					local cat = value[2]
					local sq = value[3]

					Dewdrop:AddLine(
						'text', self:TooltipSQPosition(cat,sq),
						'tooltipTitle', TooltipCheck(L["Search query: Sort"]),
						'tooltipText', L["Sort search query by assigning a number to it.\nSort sequence:\n1.) number\n2.) rarity\n3.) upper case\n4.) lower case"],
						'hasArrow', true,
						'value', {'editSearchQuery2pos', cat, sq}
					)

					Dewdrop:AddLine(
						'text', self:TooltipSQSearchName(cat,sq),
						'tooltipTitle', TooltipCheck(L["Search query: Rename search query"]),
						'tooltipText', L["Rename the name of your search query. This is the name that is shown in the menu."],
						'hasArrow', true,
						'hasEditBox', true,
						'editBoxText', sq,
						'editBoxValidateFunc', function(sqname)
							if sqname:len() >= 1 and sqname:len() <= maxBoxInputLength then
								return true
							end
						end,
						'editBoxFunc', function(sqname)
							if sqname ~= sq then
								local owCheck = ConfirmCheck(tab[cat].queries,sqname)
								local func = function()
									tab[cat].queries[sqname] = tab[cat].queries[sq]
									tab[cat].queries[sq] = nil
									Print(string_format(L["Search query '%s' (Search term: '%s' - Category: %s) RENAMED TO '%s'!"],sq,tab[cat].queries[sqname].name,cat,sqname))
								end
								if owCheck then
									local confirmTxt = string_format(L["Overwrite excisting search query '%s' in category '%s'?"],sqname,cat)
									self:ConfirmPopup(confirmTxt,func)
								else
									func()
								end
								Dewdrop:Refresh(3)
								Dewdrop:Close(4)
							end
						end
					)
					Dewdrop:AddLine(
						'text', self:TooltipSQSearchTerm(cat,sq),
						'tooltipTitle', TooltipCheck(L["Search query: Rename search term"]),
						'tooltipText', L["Rename the search term of your search query. This is the search term that is shown in the auction house searchtextbox (blank will search all)."],
						'hasArrow', true,
						'hasEditBox', true,
						'editBoxText', tab[cat].queries[sq].name,
						'editBoxFunc', function(newname)
							local oldname = tab[cat].queries[sq].name
							tab[cat].queries[sq].name = newname
							Print(string_format(L["Search term '%s' (Search query: %s - Category: %s) CHANGED TO '%s'!"],oldname,sq,cat,newname))
						end
					)
					Dewdrop:AddLine(
						'text', self:TooltipSQLevelRange(cat,sq),
						'tooltipTitle', TooltipCheck(L["Search query: Level range"]),
						'tooltipText', L["Change min. and max. level range (blank (no value) will search all)."],
						'hasArrow', true,
						'value', {'editSearchQuery2LevelRange', cat, sq}
					)
					Dewdrop:AddLine(
						'text', self:TooltipSQRarity(cat,sq),
						'tooltipTitle', TooltipCheck(L["Search query: Rarity"]),
						'tooltipText', L["Change rarity. Only selected rarity or better will show in searchresult."],
						'hasArrow', true,
						'value', {'editSearchQuery2Rarity', cat, sq}
					)
					if tab[cat].queries[sq].isUsable then
						if tab[cat].queries[sq].isUsable == 1 then
							Dewdrop:AddLine(
								'text', colorTooltipDesc..L["Usable Items"]..": "..colorTooltipYes..L["YES"],
								'tooltipTitle', TooltipCheck(L["Search query: Usable Items"]),
								'isRadio', true,
								'checked', true,
								'func', function()
									tab[cat].queries[sq].isUsable = 0
									Print(string_format(L["Usable Items for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipNo..L["NO"]..colorTooltipData))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						else
							Dewdrop:AddLine(
								'text', colorTooltipDesc..L["Usable Items"]..": "..colorTooltipNo..L["NO"],
								'tooltipTitle', TooltipCheck(L["Search query: Usable Items"]),
								'isRadio', true,
								'checked', false,
								'func', function()
									tab[cat].queries[sq].isUsable = 1
									Print(string_format(L["Usable Items for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipYes..L["YES"]..colorTooltipData))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						end
					else
						Dewdrop:AddLine(
							'text', colorTooltipDesc..L["Usable Items"]..": "..colorTooltipNo..L["NO"],
							'tooltipTitle', TooltipCheck(L["Search query: Usable Items"]),
							'isRadio', true,
							'checked', false,
							'func', function()
								tab[cat].queries[sq].isUsable = 1
								Print(string_format(L["Usable Items for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipYes..L["YES"]..colorTooltipData))
								Dewdrop:Refresh(3)
								Dewdrop:Close(4)
							end
						)
					end
					if tab[cat].queries[sq].showOnPlayer then
						if tab[cat].queries[sq].showOnPlayer == 1 then
							Dewdrop:AddLine(
								'text', colorTooltipDesc..L["Display on Character"]..": "..colorTooltipYes..L["YES"],
								'tooltipTitle', TooltipCheck(L["Search query: Display on Character"]),
								'isRadio', true,
								'checked', true,
								'func', function()
									tab[cat].queries[sq].showOnPlayer = 0
									Print(string_format(L["Display on Character for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipNo..L["NO"]..colorTooltipData))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						else
							Dewdrop:AddLine(
								'text', colorTooltipDesc..L["Display on Character"]..": "..colorTooltipNo..L["NO"],
								'tooltipTitle', TooltipCheck(L["Search query: Display on Character"]),
								'isRadio', true,
								'checked', false,
								'func', function()
									tab[cat].queries[sq].showOnPlayer = 1
									Print(string_format(L["Display on Character for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipYes..L["YES"]..colorTooltipData))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						end
					else
						Dewdrop:AddLine(
							'text', colorTooltipDesc..L["Display on Character"]..": "..colorTooltipNo..L["NO"],
							'tooltipTitle', TooltipCheck(L["Search query: Display on Character"]),
							'isRadio', true,
							'checked', false,
							'func', function()
								tab[cat].queries[sq].showOnPlayer = 1
								Print(string_format(L["Display on Character for search query '%s' (Category: %s) SET TO %s!"],sq,cat,colorTooltipYes..L["YES"]..colorTooltipData))
								Dewdrop:Refresh(3)
								Dewdrop:Close(4)
							end
						)
					end
					Dewdrop:AddLine(
						'text', self:TooltipSQFilters(cat,sq,0),
						'tooltipTitle', TooltipCheck(L["Search query: Filters"]),
						'tooltipText', L["Select a filter. Only one filter is selectable."],
						'hasArrow', true,
						'value', {'editSearchQuery2FiltersLVL1', cat, sq}
					)
					if AHsearch_Options.ttFiltersDisplay then
						if tab[cat].queries[sq].classIndex then
							Dewdrop:AddLine(
								'text', self:TooltipSQFilters(cat,sq,1),
								'tooltipTitle', "",
								'disabled', true
							)
						end
						if tab[cat].queries[sq].subclass then
							Dewdrop:AddLine(
								'text', self:TooltipSQFilters(cat,sq,2),
								'tooltipTitle', "",
								'disabled', true
							)
						end
						if tab[cat].queries[sq].invTypeIndex then
							Dewdrop:AddLine(
								'text', self:TooltipSQFilters(cat,sq,3),
								'tooltipTitle', "",
								'disabled', true
							)
						end
					else
						Dewdrop:AddLine(
							'text', self:TooltipSQFilters(cat,sq,1),
							'tooltipTitle', "",
							'disabled', true
						)
						Dewdrop:AddLine(
							'text', self:TooltipSQFilters(cat,sq,2),
							'tooltipTitle', "",
							'disabled', true
						)
						Dewdrop:AddLine(
							'text', self:TooltipSQFilters(cat,sq,3),
							'tooltipTitle', "",
							'disabled', true
						)
					end
				end

				if type(value) == "table" and value[1] == "moveSearchQuery2" then
					local oldCat = value[2]
					local query = value[3]
					for newCat,v in SortedPairs(tab) do
						if newCat == oldCat then
							Dewdrop:AddLine(
								'text', newCat,
								'tooltipTitle', TooltipCheck(""),
								'icon', tab[newCat].icon,
								'disabled', true
							)
						else
							Dewdrop:AddLine(
								'text', tab[newCat].color..newCat,
								'tooltipTitle', TooltipCheck(L["Search query: Move"]),
								'tooltipText', string_format(L["Move\nsearch query '%s'\nin category '%s'\nto category '%s'.\n\nATTENTION:\nPosition is set to 1!"],query,oldCat,newCat),
								'icon', tab[newCat].icon,
								'func', function()
									if newCat ~= oldCat then
										local owCheck = ConfirmCheck(tab[newCat].queries,query)
										local func = function()
											tab[newCat].queries[query] = tab[oldCat].queries[query]
											tab[newCat].queries[query].order = 1
											tab[oldCat].queries[query] = nil
											Print(string_format(L["Search query '%s' in category '%s' MOVED TO category '%s'."],query,oldCat,newCat))
										end
										if owCheck then
											local confirmTxt = string_format(L["Search query '%s' exists in category '%s'! Overwrite?"],query,newCat)
											self:ConfirmPopup(confirmTxt,func)
										else
											func()
										end
										Dewdrop:Refresh(2)
										Dewdrop:Refresh(3)
										Dewdrop:Close(4)
									end
								end
							)
						end
					end
				end

				if type(value) == "table" and value[1] == "editCat3pos" then
					local cat = value[2]
					local c = 1
					for k,v in SortedPairs(tab) do
						if c == tab[cat].order then
							Dewdrop:AddLine(
								'text', c,
								'tooltipTitle', "",
								'icon', "Interface\\Buttons\\UI-CheckBox-Check",
								'arg1', c,
								'func', function(value)
									tab[cat].order = value
									Print(string_format(L["ASSIGNED sort number '%s' to category '%s'!"],value,cat))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						else
							Dewdrop:AddLine(
								'text', c,
								'tooltipTitle', "",
								'arg1', c,
								'func', function(value)
									tab[cat].order = value
									Print(string_format(L["ASSIGNED sort number '%s' to category '%s'!"],value,cat))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						end
						c=c+1
					end
				end

				if type(value) == "table" and value[1] == "editCat3color" then
					local cat = value[2]
					for i=1, #categoryColors do
						if tab[cat].color == categoryColors[i] then
							Dewdrop:AddLine(
								'text', categoryColors[i]..i.." ### - "..cat,
								'tooltipTitle', "",
								'checked', true, 
								'arg1', categoryColors[i],
								'func', function(value)
									tab[cat].color = value
									Print(string_format(L["SELECTED color for category '%s': %s%s ### '%s'|r"],cat,categoryColors[i],i,cat))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						else
							Dewdrop:AddLine(
								'text', categoryColors[i]..i.." ### - "..cat,
								'tooltipTitle', "",
								'checked', false, 
								'arg1', categoryColors[i],
								'func', function(value)
									tab[cat].color = value
									Print(string_format(L["SELECTED color for category '%s': %s%s ### '%s'|r"],cat,categoryColors[i],i,cat))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						end
					end
				end

				if type(value) == "table" and value[1] == "editCat3icon" then
					local cat = value[2]
					for i=1, #categoryIcons do
						if tab[cat].icon == categoryIcons[i] then
							Dewdrop:AddLine(
								'text', "= "..L["selected"],
								'tooltipTitle', "",
								'icon', categoryIcons[i],
								'arg1', categoryIcons[i],
								'func', function(value)
									tab[cat].icon = value
									Print(string_format(L["SELECTED icon for category '%s': %s"],cat,categoryIcons[i]))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						else
							Dewdrop:AddLine(
								'icon', categoryIcons[i],
								'arg1', categoryIcons[i],
								'func', function(value)
									tab[cat].icon = value
									Print(string_format(L["SELECTED icon for category '%s': %s"],cat,categoryIcons[i]))
									Dewdrop:Refresh(2)
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
								end
							)
						end
					end
				end
			-- ----------------------------------------------------------------------
			elseif level == 5 then
			-- ----------------------------------------------------------------------
				if type(value) == "table" and value[1] == "editSearchQuery2pos" then
					local cat = value[2]
					local subTab = value[3]
					local c = 1
					for sq,v in SortedPairs(tab[cat].queries,1) do
						if c == tab[cat].queries[subTab].order then
							Dewdrop:AddLine(
								'text', c,
								'tooltipTitle', "",
								'icon', "Interface\\Buttons\\UI-CheckBox-Check",
								'arg1', c,
								'func', function(value)
									tab[cat].queries[subTab].order = value
									Print(string_format(L["ASSIGNED sort number '%s' to search query '%s' (Category: %s)!"],value,subTab,cat))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
									Dewdrop:Close(5)
								end
							)
						else
							Dewdrop:AddLine(
								'text', c,
								'tooltipTitle', "",
								'arg1', c,
								'func', function(value)
									tab[cat].queries[subTab].order = value
									Print(string_format(L["ASSIGNED sort number '%s' to search query '%s' (Category: %s)!"],value,subTab,cat))
									Dewdrop:Refresh(3)
									Dewdrop:Close(4)
									Dewdrop:Close(5)
								end
							)
						end
						c=c+1
					end
				end

				if type(value) == "table" and value[1] == "editSearchQuery2LevelRange" then
					local cat = value[2]
					local sq = value[3]
					Dewdrop:AddLine(
						'text', L["min"],
						'tooltipTitle', "",
						'hasArrow', true,
						'value', {'editSearchQuery2LevelRangeMin', cat, sq}
					)
					Dewdrop:AddLine(
						'text', L["max"],
						'tooltipTitle', "",
						'hasArrow', true,
						'value', {'editSearchQuery2LevelRangeMax', cat, sq}
					)
				end

				if type(value) == "table" and value[1] == "editSearchQuery2Rarity" then
					local cat = value[2]
					local sq = value[3]
					for i=1, #itemRarityNames do
						if i == (tab[cat].queries[sq].qualityIndex+2) then
							Dewdrop:AddLine(
								'text', itemRarityColors[i]..itemRarityNames[i],
								'tooltipTitle', "",
								'icon', "Interface\\Buttons\\UI-CheckBox-Check"
							)
						else
							Dewdrop:AddLine(
								'text', itemRarityColors[i]..itemRarityNames[i],
								'tooltipTitle', "",
								'arg1', (i-2),
								'func', function(value)
									tab[cat].queries[sq].qualityIndex = value
									Print(string_format(L["Rarity for search query '%s' (Category: %s) CHANGED TO %s!"],sq,cat,itemRarityColors[value+2]..itemRarityNames[value+2]..colorTooltipData))
									Dewdrop:Refresh(3)
									Dewdrop:Close(5)
									Dewdrop:Close(4)
								end
							)
						end
					end
				end

				if type(value) == "table" and value[1] == "editSearchQuery2FiltersLVL1" then
					local cat = value[2]
					local sq = value[3]
					local x = {}
					x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8],x[9],x[10],x[11],x[12],x[13],x[14],x[15],x[16],x[17],x[18],x[19],x[20] = GetAuctionItemClasses()
					for i=1, getn(x) do
						local a = {}
						a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8],a[9],a[10],a[11],a[12],a[13],a[14],a[15],a[16],a[17],a[18],a[19],a[20] = GetAuctionItemSubClasses(i)
						local func = function()
							tab[cat].queries[sq].classIndex = i
							tab[cat].queries[sq].subclass = nil
							tab[cat].queries[sq].subclassIndex = nil
							tab[cat].queries[sq].invTypeIndex = nil
							Print(string_format(L["Filters for search query '%s' (Category: %s) CHANGED TO %s!"],sq,cat,x[i]))
							Dewdrop:Refresh(3)
							Dewdrop:Refresh(4)
							Dewdrop:Close(5)
						end
						local func2 = function()
							tab[cat].queries[sq].classIndex = nil
							tab[cat].queries[sq].subclass = nil
							tab[cat].queries[sq].subclassIndex = nil
							tab[cat].queries[sq].invTypeIndex = nil
							Print(string_format(L["Filters for search query '%s' (Category: %s) CHANGED TO %s!"],sq,cat,"'"..L["no filters"].."'"))
							Dewdrop:Refresh(3)
							Dewdrop:Refresh(4)
							Dewdrop:Close(5)
						end
						if i == 1 then
							if tab[cat].queries[sq].classIndex then
								Dewdrop:AddLine(
									'text', L["no filters"],
									'tooltipTitle', "",
									'func', func2
								)
							else
								Dewdrop:AddLine(
									'text', L["no filters"],
									'tooltipTitle', "",
									'icon', "Interface\\Buttons\\UI-CheckBox-Check",
									'func', func2
								)
							end
						end
						if getn(a) > 0 then
							if i == tab[cat].queries[sq].classIndex then
								Dewdrop:AddLine(
									'text', x[i],
									'tooltipTitle', "",
									'icon', "Interface\\Buttons\\UI-CheckBox-Check",
									'hasArrow', true,
									'value', {'editSearchQuery2FiltersLVL2', cat, sq, i, x[i], getn(a)},
									'func', func
								)
							else
								Dewdrop:AddLine(
									'text', x[i],
									'tooltipTitle', "",
									'hasArrow', true,
									'value', {'editSearchQuery2FiltersLVL2', cat, sq, i, x[i], getn(a)},
									'func', func
								)
							end
						else
							if i == tab[cat].queries[sq].classIndex then
								Dewdrop:AddLine(
									'text', x[i],
									'tooltipTitle', "",
									'icon', "Interface\\Buttons\\UI-CheckBox-Check",
									'func', func
								)
							else
								Dewdrop:AddLine(
									'text', x[i],
									'tooltipTitle', "",
									'func', func
								)
							end
						end
					end
				end
			-- ----------------------------------------------------------------------
			elseif level == 6 then
			-- ----------------------------------------------------------------------
				if type(value) == "table" and value[1] == "editSearchQuery2LevelRangeMin" then
					local cat = value[2]
					local sq = value[3]
					local lrmin = tonumber(tab[cat].queries[sq].minLevel)
					local lrmax = tonumber(tab[cat].queries[sq].maxLevel)
					local lrsel
					if lrmax then
						if lrmax > maxItemLevel then
							lrsel = maxItemLevel
						else
							lrsel = lrmax
						end
					else
						lrsel = maxItemLevel
					end
					if tab[cat].queries[sq].minLevel == "" then
						Dewdrop:AddLine(
							'text', L["no value"],
							'tooltipTitle', "",
							'icon', "Interface\\Buttons\\UI-CheckBox-Check"
						)
					else
						Dewdrop:AddLine(
							'text', L["no value"],
							'tooltipTitle', "",
							'func', function()
								tab[cat].queries[sq].minLevel = ""
								Print(string_format(L["Min. Level for search query '%s' (Category: %s) CHANGED TO 'no value'!"],sq,cat))
								Dewdrop:Refresh(3)
								Dewdrop:Refresh(4)
								Dewdrop:Close(6)
								Dewdrop:Close(5)
							end
						)
					end
					for i=0, lrsel do
						if i == lrmin then
							Dewdrop:AddLine(
								'text', i,
								'tooltipTitle', "",
								'icon', "Interface\\Buttons\\UI-CheckBox-Check"
							)
						else
							Dewdrop:AddLine(
								'text', i,
								'tooltipTitle', "",
								'arg1', i,
								'func', function(value)
									tab[cat].queries[sq].minLevel = tostring(value)
									Print(string_format(L["Min. Level for search query '%s' (Category: %s) CHANGED TO %s!"],sq,cat,value))
									Dewdrop:Refresh(3)
									Dewdrop:Refresh(4)
									Dewdrop:Close(6)
									Dewdrop:Close(5)
								end
							)
						end
					end
				end

				if type(value) == "table" and value[1] == "editSearchQuery2LevelRangeMax" then
					local cat = value[2]
					local sq = value[3]
					local lrmin = tonumber(tab[cat].queries[sq].minLevel)
					local lrmax = tonumber(tab[cat].queries[sq].maxLevel)
					local lrsel
					if lrmin then
						if lrmin > maxItemLevel then
							lrsel = maxItemLevel
						else
							lrsel = lrmin
						end
					else
						lrsel = 0
					end
					for i=lrsel, maxItemLevel do
						if i == lrmax then
							Dewdrop:AddLine(
								'text', i,
								'tooltipTitle', "",
								'icon', "Interface\\Buttons\\UI-CheckBox-Check"
							)
						else
							Dewdrop:AddLine(
								'text', i,
								'tooltipTitle', "",
								'arg1', i,
								'func', function(value)
									tab[cat].queries[sq].maxLevel = tostring(value)
									Print(string_format(L["Max. Level for search query '%s' (Category: %s) CHANGED TO %s!"],sq,cat,value))
									Dewdrop:Refresh(3)
									Dewdrop:Refresh(4)
									Dewdrop:Close(6)
									Dewdrop:Close(5)
								end
							)
						end
					end
					if tab[cat].queries[sq].maxLevel == "" then
						Dewdrop:AddLine(
							'text', L["no value"],
							'tooltipTitle', "",
							'icon', "Interface\\Buttons\\UI-CheckBox-Check"
						)
					else
						Dewdrop:AddLine(
							'text', L["no value"],
							'tooltipTitle', "",
							'func', function()
								tab[cat].queries[sq].maxLevel = ""
								Print(string_format(L["Max. Level for search query '%s' (Category: %s) CHANGED TO 'no value'!"],sq,cat))
								Dewdrop:Refresh(3)
								Dewdrop:Refresh(4)
								Dewdrop:Close(6)
								Dewdrop:Close(5)
							end
						)
					end
				end

				if type(value) == "table" and value[1] == "editSearchQuery2FiltersLVL2" then
					local cat = value[2]
					local sq = value[3]
					local lvl1 = value[4]
					local lvl1name = value[5]
					local lvl1count = value[6]
					for i=1, lvl1count do
						local a = {}
						a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8],a[9],a[10],a[11],a[12],a[13],a[14],a[15],a[16],a[17],a[18],a[19],a[20] = GetAuctionItemSubClasses(lvl1)
						local b = {}
						b[1],b[2],b[3],b[4],b[5],b[6],b[7],b[8],b[9],b[10],b[11],b[12],b[13],b[14],b[15],b[16],b[17],b[18],b[19],b[20] = GetAuctionInvTypes(lvl1,i)
						local func = function()
							tab[cat].queries[sq].classIndex = lvl1
							tab[cat].queries[sq].subclass = "|cffffffff"..a[i].."|r"
							tab[cat].queries[sq].subclassIndex = i
							tab[cat].queries[sq].invTypeIndex = nil
							Print(string_format(L["Filters for search query '%s' (Category: %s) CHANGED TO %s-%s!"],sq,cat,lvl1name,a[i]))
							Dewdrop:Refresh(3)
							Dewdrop:Refresh(4)
							Dewdrop:Close(5)
							Dewdrop:Close(6)
						end
						if tab[cat].queries[sq].subclassIndex then
							if i == tab[cat].queries[sq].subclassIndex and tab[cat].queries[sq].classIndex == lvl1 then
								if getn(b) > 0 then
									Dewdrop:AddLine(
										'text', a[i],
										'tooltipTitle', "",
										'icon', "Interface\\Buttons\\UI-CheckBox-Check",
										'hasArrow', true,
										--'value', {'editSearchQuery2FiltersLVL3', cat, sq, lvl1, lvl1name, i, a[i], getn(b)},
										'func', func
									)
								else
									Dewdrop:AddLine(
										'text', a[i],
										'tooltipTitle', "",
										'icon', "Interface\\Buttons\\UI-CheckBox-Check",
										'func', func
									)
								end
							else
								if getn(b) > 0 then
									Dewdrop:AddLine(
										'text', a[i],
										'tooltipTitle', "",
										'hasArrow', true,
										--'value', {'editSearchQuery2FiltersLVL3', cat, sq, lvl1, lvl1name, i, a[i], getn(b)},
										'func', func
									)
								else
									Dewdrop:AddLine(
										'text', a[i]..i,
										'tooltipTitle', "",
										'func', func
									)
								end
							end
						else
							if getn(b) > 0 then
								Dewdrop:AddLine(
									'text', a[i],
									'tooltipTitle', "",
									'hasArrow', true,
									--'value', {'editSearchQuery2FiltersLVL3', cat, sq, lvl1, lvl1name, i, a[i], getn(b)},
									'func', func
								)
							else
								Dewdrop:AddLine(
									'text', a[i],
									'tooltipTitle', "",
									'func', func
								)
							end
						end
					end
				end
			-- ----------------------------------------------------------------------
			elseif level == 7 then
			-- ----------------------------------------------------------------------
				if type(value) == "table" and value[1] == "editSearchQuery2FiltersLVL3" then
					local cat = value[2]
					local sq = value[3]
					local lvl1 = value[4]
					local lvl1name = value[5]
					local lvl2 = value[6]
					local lvl2name = value[7]
					local lvl2count = value[8]
					for i=1, lvl2count do
						local a = {}
						a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8],a[9],a[10],a[11],a[12],a[13],a[14],a[15],a[16],a[17],a[18],a[19],a[20] = GetAuctionInvTypes(lvl1,lvl2)
						local func = function()
							tab[cat].queries[sq].classIndex = lvl1
							tab[cat].queries[sq].subclass = "|cffffffff"..lvl2name.."|r"
							tab[cat].queries[sq].subclassIndex = lvl2
							tab[cat].queries[sq].invTypeIndex = i
							Print(string_format(L["Filters for search query '%s' (Category: %s) CHANGED TO %s-%s-%s!"],sq,cat,lvl1name,lvl2name,getglobal(a[i])))
							Dewdrop:Refresh(3)
							Dewdrop:Refresh(4)
							Dewdrop:Close(5)
							Dewdrop:Close(6)
							Dewdrop:Close(7)
						end
						if tab[cat].queries[sq].invTypeIndex then
							if i == tab[cat].queries[sq].invTypeIndex and tab[cat].queries[sq].subclassIndex == lvl2 then
								Dewdrop:AddLine(
									'text', getglobal(a[i]),
									'tooltipTitle', "",
									'icon', "Interface\\Buttons\\UI-CheckBox-Check",
									'func', func
								)
							else
								Dewdrop:AddLine(
									'text', getglobal(a[i]),
									'tooltipTitle', "",
									'func', func
								)
							end
						else
							Dewdrop:AddLine(
								'text', getglobal(a[i]),
								'tooltipTitle', "",
								'func', func
							)
						end
					end
				end

			end
		end
	)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:Search(cat,sq)
	-- save data for cycle search
	cylceCat = cat
	cycleSQ = sq
	-- load fields in AuctionFrame
	local tab = AHsearch_Data[cat].queries[sq]
	BrowseName:SetText(tab.name)
	BrowseMinLevel:SetText(tab.minLevel)
	BrowseMaxLevel:SetText(tab.maxLevel)
	AuctionFrameBrowse.selectedInvtypeIndex = tab.invTypeIndex
	AuctionFrameBrowse.selectedClassIndex = tab.classIndex
	AuctionFrameBrowse.selectedClass = CLASS_FILTERS[tab.classIndex]
	AuctionFrameBrowse.selectedSubclassIndex = tab.subclassIndex
	AuctionFrameBrowse.selectedSubclass = tab.subclass
	IsUsableCheckButton:SetChecked(tab.isUsable)
	ShowOnPlayerCheckButton:SetChecked(tab.showOnPlayer)
	AUCTION_DISPLAY_ON_CHARACTER = tostring(tab.showOnPlayer)
	UIDropDownMenu_SetSelectedValue(BrowseDropDown, tab.qualityIndex)
	-- update AuctionFrame
	UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
	AuctionFrameFilters_Update()
	UIDropDownMenu_Refresh(BrowseDropDown, false)
	-- search
	AuctionFrameBrowse_Search()
	Print(colorAction..string.upper(L["Search"]).."|r "..QualityColored(cat,sq).."|r: '"..colorBase..tab.name.."|r' ("..AHsearch_Data[cat].color..cat.."|r)")
	Dewdrop:Close(1)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CycleSearch(dir)
	if not cycle then return end
	-- if no category is selected use the first/last category and the first/last search entry
	if not cylceCat then
		-- first/last category
		local temp = {}
		local x = 1
		for cat,v in SortedPairs(AHsearch_Data) do
			if SQCount(cat) > 0 then
				temp[x] = cat
				x = x + 1
			end
		end
		if x == 1 then return end -- return if no search queries was saved
		if dir == 1 then
			cylceCat = temp[1]
		else
			cylceCat = temp[#temp]
		end
		temp = nil
		-- first/last search entry
		temp = {}
		x = 1
		for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
			temp[x] = sq
			x = x + 1
		end
		if dir == 1 then
			cycleSQ = temp[1]
		else
			cycleSQ = temp[#temp]
		end
		temp = nil
		--DEFAULT_CHAT_FRAME:AddMessage("cylceCat = "..cylceCat.." cycleSQ = "..cycleSQ)
		AHsearch:Search(cylceCat,cycleSQ)
	else
		local newSQ
		local temp = {}
		local x = 1
		for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
			temp[x] = sq
			x = x + 1
		end

		local tempCount = #temp

		local stop
		for i=1, tempCount do
			if temp[i] == cycleSQ then
				stop = i
				break
			end
		end

		-- next search START
		if dir == 1 then
			if stop == tempCount then
				-- switch to next category START
				local temp2 = {}
				local x = 1
				for cat,v in SortedPairs(AHsearch_Data) do
					if SQCount(cat) > 0 then
						temp2[x] = cat
						x = x + 1
					end
				end
				
				local stop2
				for i=1, #temp2 do
					if temp2[i] == cylceCat then
						stop2 = i
						break
					end
				end
				
				if stop2 == #temp2 then -- if its the last cat switch to first cat
					cylceCat = temp2[1]
					
					local temp3 = {}
					local x = 1
					for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
						temp3[x] = sq
						x = x + 1
					end
					
					newSQ = temp3[1]
					temp3 = nil
				else -- otherwise just switch to next cat
					cylceCat = temp2[stop2+1]

					local temp3 = {}
					local x = 1
					for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
						temp3[x] = sq
						x = x + 1
					end

					newSQ = temp3[1]
					temp3 = nil
				end
				temp2 = nil
				-- switch to next category END
			else
				newSQ = temp[stop+1]
			end
		-- next search END
		-- previous search START
		else
			if stop == 1 then
				-- switch to previous category START
				local temp2 = {}
				local x = 1
				for cat,v in SortedPairs(AHsearch_Data) do
					if SQCount(cat) > 0 then
						temp2[x] = cat
						x = x + 1
					end
				end

				local stop2
				for i=1, #temp2 do
					if temp2[i] == cylceCat then
						stop2 = i
						break
					end
				end

				if stop2 == 1 then -- if its the first cat switch to last cat
					cylceCat = temp2[#temp2]

					local temp3 = {}
					local x = 1
					for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
						temp3[x] = sq
						x = x + 1
					end
					
					newSQ = temp3[#temp3]
					temp3 = nil
				else -- otherwise just switch to previous cat
					cylceCat = temp2[stop2-1]

					local temp3 = {}
					local x = 1
					for sq,v in SortedPairs(AHsearch_Data[cylceCat].queries,1) do
						temp3[x] = sq
						x = x + 1
					end

					newSQ = temp3[#temp3]
					temp3 = nil
				end
				-- switch to previous category END
			else
				newSQ = temp[stop-1]
			end
		end
		-- previous search END

		temp = nil

		--DEFAULT_CHAT_FRAME:AddMessage("cylceCat = "..cylceCat.." cycleSQ = "..newSQ)
		AHsearch:Search(cylceCat,newSQ)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:RandomSearch()
	if not cycle then return end

	local temp = {}
	local x = 1
	for cat,v in pairs(AHsearch_Data) do
		for sq,w in pairs(AHsearch_Data[cat].queries) do
			temp[x] = {}
			temp[x].c = cat
			temp[x].s = sq
			x = x + 1
		end
	end
	if x == 1 then return end -- return if no search queries was saved

	local rnd = math.random(#temp)
	cylceCat = temp[rnd].c
	cycleSQ = temp[rnd].s
	temp = nil

	AHsearch:Search(cylceCat,cycleSQ)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:SaveSearchInCategory(cat,sq)
	local lastSearch = { 
		name = BrowseName:GetText(), 
		order = 1,
		minLevel = BrowseMinLevel:GetText(), 
		maxLevel = BrowseMaxLevel:GetText(), 
		invTypeIndex = AuctionFrameBrowse.selectedInvtypeIndex, 
		classIndex = AuctionFrameBrowse.selectedClassIndex,
		subclassIndex = AuctionFrameBrowse.selectedSubclassIndex,
		subclass = AuctionFrameBrowse.selectedSubclass,
		isUsable = IsUsableCheckButton:GetChecked(),
		showOnPlayer = ShowOnPlayerCheckButton:GetChecked()
	}

	if not UIDropDownMenu_GetSelectedValue(BrowseDropDown) then
		lastSearch.qualityIndex = -1
		UIDropDownMenu_SetSelectedValue(BrowseDropDown, -1)
		UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
		AuctionFrameFilters_Update()
		UIDropDownMenu_Refresh(BrowseDropDown, false)
	else
		lastSearch.qualityIndex = UIDropDownMenu_GetSelectedValue(BrowseDropDown)
	end

	AHsearch_Data[cat].queries[sq] = {}
	AHsearch_Data[cat].queries[sq] = lastSearch
	Print(string_format(L["Search query '%s' SAVED in category '%s'!"],sq,cat))
	Dewdrop:Close(1)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Search Query Tooltip
-- --------------------------
function AHsearch:SQtooltip(cat,sq)
	if not AHsearch_Options.ttDisplay then return end
	local s = ""
	s = s..self:TooltipCATName(cat)
	s = s.."\n"..self:TooltipSQPosition(cat,sq)
	s = s.."\n"..self:TooltipSQSearchName(cat,sq)
	s = s.."\n"..self:TooltipSQSearchTerm(cat,sq)
	s = s.."\n"..self:TooltipSQLevelRange(cat,sq)
	s = s.."\n"..self:TooltipSQRarity(cat,sq)
	s = s.."\n"..self:TooltipSQUsableItems(cat,sq)
	s = s.."\n"..self:TooltipSQPreview(cat,sq)
	
	if AHsearch_Options.ttFiltersDisplay then
		if AHsearch_Data[cat].queries[sq].classIndex then
			if AHsearch_Data[cat].queries[sq].subclass then
				if AHsearch_Data[cat].queries[sq].invTypeIndex then
					s = s.."\n"..self:TooltipSQFilters(cat,sq,0).."\n"..self:TooltipSQFilters(cat,sq,1).."\n"..self:TooltipSQFilters(cat,sq,2).."\n"..self:TooltipSQFilters(cat,sq,3)
				else
					s = s.."\n"..self:TooltipSQFilters(cat,sq,0).."\n"..self:TooltipSQFilters(cat,sq,1).."\n"..self:TooltipSQFilters(cat,sq,2)
				end
			else
				s = s.."\n"..self:TooltipSQFilters(cat,sq,0).."\n"..self:TooltipSQFilters(cat,sq,1)
			end
		else
			s = s.."\n"..self:TooltipSQFilters(cat,sq,0)
		end
	else
		s = s.."\n"..self:TooltipSQFilters(cat,sq,0).."\n"..self:TooltipSQFilters(cat,sq,1).."\n"..self:TooltipSQFilters(cat,sq,2).."\n"..self:TooltipSQFilters(cat,sq,3)
	end
	return s
end

function AHsearch:TooltipSQPosition(cat,sq)
	return colorTooltipDesc..L["Position"]..": "..colorTooltipData..AHsearch_Data[cat].queries[sq].order
end

function AHsearch:TooltipSQSearchName(cat,sq)
	return colorTooltipDesc..L["Search query"]..": "..itemRarityColors[AHsearch_Data[cat].queries[sq].qualityIndex+2]..sq
end

function AHsearch:TooltipSQSearchTerm(cat,sq)
	local s = AHsearch_Data[cat].queries[sq].name
	if s == "" then
		s = "-"
	end
	return colorTooltipDesc..L["Search term"]..": "..colorTooltipData..s
end

function AHsearch:TooltipSQLevelRange(cat,sq)
	local tt_lr
	local tt_lrmin = AHsearch_Data[cat].queries[sq].minLevel
	local tt_lrmax = AHsearch_Data[cat].queries[sq].maxLevel
	if tt_lrmin == "" and tt_lrmax == "" then
		tt_lr = "-"
	end
	if tt_lrmin == "" and tt_lrmax ~= "" then
		tt_lr = "-"..tt_lrmax
	end
	if tt_lrmin ~= "" and tt_lrmax == "" then
		tt_lr = tt_lrmin.."+"
	end
	if tt_lrmin ~= "" and tt_lrmax ~= "" then
		if tt_lrmin == tt_lrmax then
			tt_lr = tt_lrmin
		else
			tt_lr = tt_lrmin.."-"..tt_lrmax
		end
	end
	return colorTooltipDesc..L["Level Range"]..": "..colorTooltipData..tt_lr
end

function AHsearch:TooltipSQRarity(cat,sq)
	local tt_r = AHsearch_Data[cat].queries[sq].qualityIndex+2
	local tt_rb = ""
	if tt_r > 1 and tt_r < 6 then
		tt_rb = colorTooltipData..L[" and better"]
	end 
	return colorTooltipDesc..L["Rarity"]..": "..itemRarityColors[tt_r]..itemRarityNames[tt_r]..tt_rb
end

function AHsearch:TooltipSQUsableItems(cat,sq)
	if AHsearch_Data[cat].queries[sq].isUsable == 1 then
		return colorTooltipDesc..L["Usable Items"]..": "..colorTooltipYes..L["YES"]
	else
		return colorTooltipDesc..L["Usable Items"]..": "..colorTooltipNo..L["NO"]
	end
end

function AHsearch:TooltipSQPreview(cat,sq)
	if AHsearch_Data[cat].queries[sq].showOnPlayer == 1 then
		return colorTooltipDesc..L["Display on Character"]..": "..colorTooltipYes..L["YES"]
	else
		return colorTooltipDesc..L["Display on Character"]..": "..colorTooltipNo..L["NO"]
	end
end

function AHsearch:TooltipSQFilters(cat,sq,lvl)
	local p1
	local p2
	local p3
	local ttf_c = CLASS_FILTERS[AHsearch_Data[cat].queries[sq].classIndex]
	local ttf_sc = AHsearch_Data[cat].queries[sq].subclass
	local ttf_iTI = AHsearch_Data[cat].queries[sq].invTypeIndex
	if ttf_c then
		if ttf_sc then
			if ttf_iTI then
				p1 = colorTooltipData.."1.) "..ttf_c
				p2 = colorTooltipData.."2.) "..ttf_sc
				p3 = colorTooltipData.."3.) disabled"
				-- disabled
				--local a = {}
				--a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8],a[9],a[10],a[11],a[12],a[13],a[14],a[15],a[16],a[17],a[18],a[19],a[20] = GetAuctionInvTypes(AHsearch_Data[cat].queries[sq].classIndex, AHsearch_Data[cat].queries[sq].subclassIndex)
				--if getglobal(a[ttf_iTI]) then
				--	p3 = colorTooltipData.."3.) "..getglobal(a[ttf_iTI])
				--else
				--	p3 = colorTooltipData.."3.) sub-cat no longer available!!!"
				--end
			else
				p1 = colorTooltipData.."1.) "..ttf_c
				p2 = colorTooltipData.."2.) "..ttf_sc
				p3 = colorTooltipData.."3.) -"
			end
		else
			p1 = colorTooltipData.."1.) "..ttf_c
			p2 = colorTooltipData.."2.) -"
			p3 = colorTooltipData.."3.) -"
		end
	else
		p1 = colorTooltipData.."1.) -"
		p2 = colorTooltipData.."2.) -"
		p3 = colorTooltipData.."3.) -"
	end

	if lvl == 0 then
		if not ttf_c then
			return colorTooltipDesc..L["Filters"]..": "..colorTooltipData..L["no filters"]
		else
			return colorTooltipDesc..L["Filters"]..":"
		end
	end
	if lvl == 1 then
		return p1
	end
	if lvl == 2 then
		return p2
	end
	if lvl == 3 then
		return p3
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Category Tooltip
-- --------------------------
function AHsearch:CATtooltip(cat)
	if not AHsearch_Options.ttDisplay then return end
	local s = ""
	s = s..self:TooltipCATName(cat)
	s = s.."\n"..self:TooltipCATPosition(cat)
	s = s.."\n"..self:TooltipCATColor(cat)
	s = s.."\n"..self:TooltipCATIcon(cat)
	return s
end

function AHsearch:TooltipCATName(cat)
	return colorTooltipDesc..L["Category"]..": "..AHsearch_Data[cat].color..cat
end

function AHsearch:TooltipCATPosition(cat)
	return colorTooltipDesc..L["Position"]..": "..colorTooltipData..AHsearch_Data[cat].order
end

function AHsearch:TooltipCATColor(cat)
	return colorTooltipDesc..L["Color"]..": "..AHsearch_Data[cat].color.."###"
end

function AHsearch:TooltipCATIcon(cat)
	local x = colorTooltipDesc..L["Icon"]..": "..colorTooltipNo..L["NO"]
	if AHsearch_Data[cat].icon ~= "" then
		x = colorTooltipDesc..L["Icon"]..": "..colorTooltipYes..L["YES"]
	end
	return x
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfirmPopup(message, func, ...)
	if not StaticPopupDialogs["AHSEARCH_CONFIRM_DIALOG"] then
		StaticPopupDialogs["AHSEARCH_CONFIRM_DIALOG"] = {}
	end
	local t = StaticPopupDialogs["AHSEARCH_CONFIRM_DIALOG"]
	for k in pairs(t) do
		t[k] = nil
	end
	t.text = message
	t.button1 = ACCEPT or "Accept"
	t.button2 = CANCEL or "Cancel"
	t.OnAccept = function()
		func(unpack(t))
	end
	for i = 1, select('#', ...) do
		t[i] = select(i, ...)
	end
	t.timeout = 0
	t.whileDead = 1
	t.hideOnEscape = 1
	
	Dewdrop:Close()
	StaticPopup_Show("AHSEARCH_CONFIRM_DIALOG")
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:HelpOutput()
	local a = AHsearch_Options.chatOutput
	AHsearch_Options.chatOutput = true
	Print(colorBase.."-- "..L["Help"].." -----------------|r")	
	Print(string_format(L["%sStep 1 or 2%s: Right-click 'AHsearch Button' and create category ('Category: Create')."],colorAction,"|r"))
	Print(string_format(L["%sStep 1 or 2%s: Set up your search criteria (you don't have to click the search button)."],colorAction,"|r"))
	Print(string_format(L["%sStep 3%s: Right-click 'AHsearch Button' and choose a category in 'Search query: Save'."],colorAction,"|r"))
	Print(string_format(L["%sStep 4%s: Give your search a name. Press Enter."],colorAction,"|r"))
	Print(string_format(L["%sStep 5%s: Left-click 'AHsearch Button' for your search queries."],colorAction,"|r"))
	Print(colorBase.."-- "..L["Help"].." -----------------|r")
	Print(string_format(L["Key Bindings for next/previous/random search: %sESC -> Key Bindings -> AHsearch%s"],colorAction,"|r"))
	Print(string_format(L["Maximum text length for categories and search queries: %s%s characters%s"],colorAction,maxBoxInputLength,"|r"))
	Print(L["Color coding for saved search queries:"])
	Print(itemRarityColors[1].."#"..itemRarityNames[1].."#"..
		" "..itemRarityColors[2].."#"..itemRarityNames[2].."#"..
		" "..itemRarityColors[3].."#"..itemRarityNames[3].."#"..
		" "..itemRarityColors[4].."#"..itemRarityNames[4].."#"..
		" "..itemRarityColors[5].."#"..itemRarityNames[5].."#"..
		" "..itemRarityColors[6].."#"..itemRarityNames[6].."#")
	AHsearch_Options.chatOutput = a
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:OnEvent()
	if event == "AUCTION_HOUSE_SHOW" then
		if not AHsearchFrame then
			AHsearch:CreateAHsearchButton()
			AHsearch:CreateAHsearchCycleButtons()
			AHsearch:CreatePullDownOptionsMenu()
		end
		cycle = true
	elseif event == "AUCTION_HOUSE_CLOSED" then
		Dewdrop:Close(1)
		cycle = false
		cylceCat = false
	elseif event == "ADDON_LOADED" then
		DBcheck()
		AHsearch:InitOptions()
		AHsearch:UnregisterEvent("ADDON_LOADED")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AHsearch:RegisterEvent("ADDON_LOADED")
AHsearch:RegisterEvent("AUCTION_HOUSE_SHOW")
AHsearch:RegisterEvent("AUCTION_HOUSE_CLOSED")
AHsearch:SetScript("OnEvent", AHsearch.OnEvent) -- start the Mystical Machine